use log::info;
use regex::Regex;
use scraper::{Html, Selector};
use std::io;
use std::path::Path;
use tokio::fs::File as TokioFile;
use tokio::io::AsyncReadExt;
use url::Url;

pub async fn get_content(link: &str) -> Result<String, reqwest::Error> {
    info!("Requesting page from SVT: {}", link);

    let resp = reqwest::get(link).await?;
    let content = resp.text().await?;

    info!("Page retrieved successfully");
    Ok(content)
}

pub fn get_relative_links(content: &str) -> Vec<String> {
    info!("Parsing content to find links");

    let fragment = Html::parse_document(content);
    let selector = Selector::parse("a[href]").unwrap();
    let mut hrefs: Vec<String> = fragment
        .select(&selector)
        .filter_map(|element| element.value().attr("href"))
        .filter(|href| href.ends_with("info=visa"))
        .map(|href| href.to_string())
        .collect();

    hrefs.sort_by_key(|url| {
        let re = Regex::new(r"(\d+)").unwrap();
        re.captures(url.split('/').last().unwrap())
            .and_then(|cap| cap.get(1).map(|m| m.as_str().parse::<usize>().unwrap()))
            .unwrap_or(usize::MAX)
    });

    info!("Found and sorted {} links", hrefs.len());
    hrefs
}

pub fn get_base_url(url: &str) -> Result<String, url::ParseError> {
    info!("Extracting base URL from: {}", url);
    let parsed_url = Url::parse(url)?;
    let base_url = format!(
        "{}://{}",
        parsed_url.scheme(),
        parsed_url.host_str().unwrap()
    );
    info!("Base URL is: {}", base_url);
    Ok(base_url)
}

async fn get_content_tmp() -> Result<String, io::Error> {
    info!("Reading content from temp.html");
    let path = Path::new("./temp.html");
    let mut file = TokioFile::open(path).await?;
    let mut content = String::new();
    file.read_to_string(&mut content).await?;
    Ok(content)
}
